@@ -15,19 +15,25 @@
 
 """Storage backend for S3 or Storage Servers that follow the S3 Protocol"""
 
+from concurrent import futures
 import io
 import logging
 import math
 import re
 import urllib
 
-from boto3 import session as boto_session
-from botocore import client as boto_client
-from botocore import exceptions as boto_exceptions
-from botocore import utils as boto_utils
-import eventlet
+try:
+    from boto3 import session as boto_session
+    from botocore import client as boto_client
+    from botocore import exceptions as boto_exceptions
+    from botocore import utils as boto_utils
+except ImportError:
+    boto_session = None
+    boto_client = None
+    boto_exceptions = None
+    boto_utils = None
+
 from oslo_config import cfg
-from oslo_utils import encodeutils
 from oslo_utils import units
 
 import glance_store
@@ -64,6 +70,22 @@ Related Options:
     * s3_store_access_key
     * s3_store_secret_key
 
+"""),
+    cfg.StrOpt('s3_store_region_name',
+               default='',
+               help="""
+The S3 region name.
+
+This parameter will set the region_name used by boto.
+If this parameter is not set, we we will try to compute it from the
+s3_store_host.
+
+Possible values:
+    * A valid region name
+
+Related Options:
+    * s3_store_host
+
 """),
     cfg.StrOpt('s3_store_access_key',
                secret=True,
@@ -157,6 +179,7 @@ Related Options:
 """),
     cfg.IntOpt('s3_store_large_object_size',
                default=DEFAULT_LARGE_OBJECT_SIZE,
+               min=0,
                help="""
 What size, in MB, should S3 start chunking image files and do a multipart
 upload in S3.
@@ -167,7 +190,7 @@ upload the image to S3 as is or to split it (Multipart Upload).
 Note: You can only split up to 10,000 images.
 
 Possible values:
-    * Any positive integer value
+    * Any positive integer value or zero
 
 Related Options:
     * s3_store_large_object_chunk_size
@@ -207,7 +230,18 @@ Related Options:
     * s3_store_large_object_size
     * s3_store_large_object_chunk_size
 
-""")
+"""),
+    cfg.StrOpt('s3_store_cacert',
+               default='',
+               help="""
+The path to the CA cert bundle to use. The default value (an empty string)
+forces the use of the default CA cert bundle used by botocore.
+
+Possible values:
+    * A path to the CA cert bundle to use
+    * An empty string to use the default CA cert bundle used by botocore
+
+"""),
 ]
 
 
@@ -374,7 +408,14 @@ class Store(glance_store.driver.Store):
         this method. If the store was not able to successfully configure
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
+        if boto_session is None:
+            reason = _("boto3 or botocore is not available.")
+            LOG.error(reason)
+            raise exceptions.BadStoreConfiguration(store_name="s3",
+                                                   reason=reason)
+
         self.s3_host = self._option_get('s3_store_host')
+        self.region_name = self._option_get('s3_store_region_name')
         self.access_key = self._option_get('s3_store_access_key')
         self.secret_key = self._option_get('s3_store_secret_key')
         self.bucket = self._option_get('s3_store_bucket')
@@ -424,7 +465,8 @@ class Store(glance_store.driver.Store):
 
     def _option_get(self, param):
         if self.backend_group:
-            store_conf = getattr(self.conf, self.backend_group)
+            store_conf = glance_store.driver.BackendGroupConfiguration(
+                self.OPTIONS, self.backend_group, conf=self.conf)
         else:
             store_conf = self.conf.glance_store
 
@@ -432,6 +474,12 @@ class Store(glance_store.driver.Store):
         if not result:
             if param == 's3_store_create_bucket_on_put':
                 return result
+            if param == 's3_store_large_object_size':
+                return result
+            if param == 's3_store_region_name':
+                return result
+            if param == 's3_store_cacert':
+                return result
             reason = _("Could not find %s in configuration options.") % param
             LOG.error(reason)
             raise exceptions.BadStoreConfiguration(store_name="s3",
@@ -460,16 +508,22 @@ class Store(glance_store.driver.Store):
             raise boto_exceptions.InvalidDNSNameError(bucket_name=bucket_name)
 
         region_name, endpoint_url = None, None
-        if location:
+        if self.region_name:
+            region_name = self.region_name
+            endpoint_url = s3_host
+        elif location:
             region_name = location
         else:
             endpoint_url = s3_host
 
-        return session.client(service_name='s3',
-                              endpoint_url=endpoint_url,
-                              region_name=region_name,
-                              use_ssl=(loc.scheme == 's3+https'),
-                              config=config)
+        store_cacert = self._option_get('s3_store_cacert')
+        return session.client(
+            service_name='s3',
+            endpoint_url=endpoint_url,
+            region_name=region_name,
+            use_ssl=(loc.scheme == 's3+https'),
+            verify=None if store_cacert == '' else store_cacert,
+            config=config)
 
     def _operation_set(self, loc):
         """Objects and variables frequently used when operating S3 are
@@ -504,10 +558,8 @@ class Store(glance_store.driver.Store):
         key = s3_client.get_object(Bucket=bucket, Key=key)
 
         LOG.debug("Retrieved image object from S3 using s3_host=%(s3_host)s, "
-                  "access_key=%(accesskey)s, bucket=%(bucket)s, "
-                  "key=%(key)s)",
-                  {'s3_host': loc.s3serviceurl, 'accesskey': loc.accesskey,
-                   'bucket': bucket, 'key': key})
+                  "bucket=%(bucket)s key=%(key)s)",
+                  {'s3_host': loc.s3serviceurl, 'bucket': bucket, 'key': key})
 
         cs = self.READ_CHUNKSIZE
 
@@ -578,7 +630,8 @@ class Store(glance_store.driver.Store):
             if self._option_get('s3_store_create_bucket_on_put'):
                 self._create_bucket(s3_client,
                                     self._option_get('s3_store_host'),
-                                    bucket)
+                                    bucket,
+                                    self._option_get('s3_store_region_name'))
             else:
                 msg = (_("The bucket %s does not exist in "
                          "S3. Please set the "
@@ -587,10 +640,8 @@ class Store(glance_store.driver.Store):
                 raise glance_store.BackendException(msg)
 
         LOG.debug("Adding image object to S3 using (s3_host=%(s3_host)s, "
-                  "access_key=%(access_key)s, bucket=%(bucket)s, "
-                  "key=%(key)s)",
-                  {'s3_host': self.s3_host, 'access_key': loc.accesskey,
-                   'bucket': bucket, 'key': key})
+                  "bucket=%(bucket)s, key=%(key)s)",
+                  {'s3_host': self.s3_host, 'bucket': bucket, 'key': key})
 
         if not self._object_exists(s3_client, bucket, key):
             if image_size < self.s3_store_large_object_size:
@@ -600,7 +651,8 @@ class Store(glance_store.driver.Store):
                                             key=key,
                                             loc=loc,
                                             hashing_algo=hashing_algo,
-                                            verifier=verifier)
+                                            verifier=verifier,
+                                            image_size=image_size)
 
             return self._add_multipart(s3_client=s3_client,
                                        image_file=image_file,
@@ -615,7 +667,7 @@ class Store(glance_store.driver.Store):
         raise exceptions.Duplicate(image=key)
 
     def _add_singlepart(self, s3_client, image_file, bucket, key, loc,
-                        hashing_algo, verifier):
+                        hashing_algo, verifier, image_size):
         """Stores an image file with a single part upload to S3 backend.
 
         :param s3_client: An object with credentials to connect to S3
@@ -626,6 +678,8 @@ class Store(glance_store.driver.Store):
                     from glance_store.location.get_location_from_uri()
         :param hashing_algo: A hashlib algorithm identifier (string)
         :param verifier: An object used to verify signatures for images
+        :param image_size: The size of the image data to write,
+                           in bytes
         :returns: tuple of: (1) URL in backing store, (2) bytes written,
                   (3) checksum, (4) multihash value, and (5) a dictionary
                   with storage system specific information
@@ -633,15 +687,28 @@ class Store(glance_store.driver.Store):
         os_hash_value = utils.get_hasher(hashing_algo, False)
         checksum = utils.get_hasher('md5', False)
         image_data = b''
-        image_size = 0
+        total_bytes = 0
         for chunk in utils.chunkreadable(image_file, self.WRITE_CHUNKSIZE):
+            total_bytes += len(chunk)
+            if image_size != 0 and total_bytes > image_size:
+                # Size exceeded, abort upload
+                raise glance_store.Invalid(
+                    _("Size exceeds: expected %(expected)d bytes, "
+                      "got %(actual)d bytes") %
+                    {'expected': image_size, 'actual': total_bytes})
             image_data += chunk
-            image_size += len(chunk)
             os_hash_value.update(chunk)
             checksum.update(chunk)
             if verifier:
                 verifier.update(chunk)
 
+        # Final size check after reading all chunks
+        if image_size != 0 and total_bytes != image_size:
+            raise glance_store.Invalid(
+                _("Size mismatch: expected %(expected)d bytes, "
+                  "got %(actual)d bytes") %
+                {'expected': image_size, 'actual': total_bytes})
+
         s3_client.put_object(Body=image_data,
                              Bucket=bucket,
                              Key=key)
@@ -655,9 +722,9 @@ class Store(glance_store.driver.Store):
 
         LOG.debug("Wrote %(size)d bytes to S3 key named %(key)s "
                   "with checksum %(checksum)s",
-                  {'size': image_size, 'key': key, 'checksum': checksum_hex})
+                  {'size': total_bytes, 'key': key, 'checksum': checksum_hex})
 
-        return loc.get_uri(), image_size, checksum_hex, hash_hex, metadata
+        return loc.get_uri(), total_bytes, checksum_hex, hash_hex, metadata
 
     def _add_multipart(self, s3_client, image_file, image_size, bucket,
                        key, loc, hashing_algo, verifier):
@@ -678,71 +745,98 @@ class Store(glance_store.driver.Store):
         os_hash_value = utils.get_hasher(hashing_algo, False)
         checksum = utils.get_hasher('md5', False)
         pool_size = self.s3_store_thread_pools
-        pool = eventlet.greenpool.GreenPool(size=pool_size)
-        mpu = s3_client.create_multipart_upload(Bucket=bucket, Key=key)
-        upload_id = mpu['UploadId']
-        LOG.debug("Multipart initiate key=%(key)s, UploadId=%(UploadId)s",
-                  {'key': key, 'UploadId': upload_id})
-        cstart = 0
-        plist = []
+        # Replace eventlet.GreenPool with ThreadPoolExecutor
+        with futures.ThreadPoolExecutor(
+                max_workers=pool_size) as executor:
+            # Create a list to store the futures
+            futures_list = []
+            mpu = s3_client.create_multipart_upload(Bucket=bucket, Key=key)
+            upload_id = mpu['UploadId']
+            LOG.debug("Multipart initiate key=%(key)s, UploadId=%(UploadId)s",
+                      {'key': key, 'UploadId': upload_id})
+            cstart = 0
+            plist = []
 
-        chunk_size = int(math.ceil(float(image_size) / MAX_PART_NUM))
-        write_chunk_size = max(self.s3_store_large_object_chunk_size,
-                               chunk_size)
-        it = utils.chunkreadable(image_file, self.WRITE_CHUNKSIZE)
-        buffered_chunk = b''
-        while True:
-            try:
-                buffered_clen = len(buffered_chunk)
-                if buffered_clen < write_chunk_size:
-                    # keep reading data
-                    read_chunk = next(it)
-                    buffered_chunk += read_chunk
-                    continue
-                else:
-                    write_chunk = buffered_chunk[:write_chunk_size]
-                    remained_data = buffered_chunk[write_chunk_size:]
-                    os_hash_value.update(write_chunk)
-                    checksum.update(write_chunk)
-                    if verifier:
-                        verifier.update(write_chunk)
-                    fp = io.BytesIO(write_chunk)
-                    fp.seek(0)
-                    part = UploadPart(mpu, fp, cstart + 1, len(write_chunk))
-                    pool.spawn_n(run_upload, s3_client, bucket, key, part)
-                    plist.append(part)
-                    cstart += 1
-                    buffered_chunk = remained_data
-            except StopIteration:
-                if len(buffered_chunk) > 0:
-                    # Write the last chunk data
-                    write_chunk = buffered_chunk
-                    os_hash_value.update(write_chunk)
-                    checksum.update(write_chunk)
-                    if verifier:
-                        verifier.update(write_chunk)
-                    fp = io.BytesIO(write_chunk)
-                    fp.seek(0)
-                    part = UploadPart(mpu, fp, cstart + 1, len(write_chunk))
-                    pool.spawn_n(run_upload, s3_client, bucket, key, part)
-                    plist.append(part)
-                break
+            chunk_size = int(math.ceil(float(image_size) / MAX_PART_NUM))
+            write_chunk_size = max(self.s3_store_large_object_chunk_size,
+                                   chunk_size)
+            it = utils.chunkreadable(image_file, self.WRITE_CHUNKSIZE)
+            buffered_chunk = b''
+            total_bytes = 0
+            while True:
+                try:
+                    buffered_clen = len(buffered_chunk)
+                    if buffered_clen < write_chunk_size:
+                        # keep reading data
+                        read_chunk = next(it)
+                        buffered_chunk += read_chunk
+                        continue
+                    else:
+                        write_chunk = buffered_chunk[:write_chunk_size]
+                        remained_data = buffered_chunk[write_chunk_size:]
+                        total_bytes += len(write_chunk)
+                        if image_size != 0 and total_bytes > image_size:
+                            # Abort multipart upload immediately
+                            s3_client.abort_multipart_upload(
+                                Bucket=bucket, Key=key, UploadId=upload_id)
+                            raise glance_store.Invalid(
+                                _("Size exceeds: expected "
+                                  "%(expected)d "
+                                  "bytes, got %(actual)d bytes") %
+                                {'expected': image_size,
+                                 'actual': total_bytes})
+                        os_hash_value.update(write_chunk)
+                        checksum.update(write_chunk)
+                        if verifier:
+                            verifier.update(write_chunk)
+                        fp = io.BytesIO(write_chunk)
+                        fp.seek(0)
+                        part = UploadPart(
+                            mpu, fp, cstart + 1, len(write_chunk))
+                        # Spawn thread to upload part
+                        futures_list.append(executor.submit(
+                            run_upload, s3_client, bucket, key, part))
+                        plist.append(part)
+                        cstart += 1
+                        buffered_chunk = remained_data
+                except StopIteration:
+                    if len(buffered_chunk) > 0:
+                        # Write the last chunk data
+                        write_chunk = buffered_chunk
+                        total_bytes += len(write_chunk)
+                        os_hash_value.update(write_chunk)
+                        checksum.update(write_chunk)
+                        if verifier:
+                            verifier.update(write_chunk)
+                        fp = io.BytesIO(write_chunk)
+                        fp.seek(0)
+                        part = UploadPart(
+                            mpu, fp, cstart + 1, len(write_chunk))
+                        futures_list.append(executor.submit(
+                            run_upload, s3_client, bucket, key, part))
+                        plist.append(part)
+                    break
 
-        pedict = {}
-        total_size = 0
-        pool.waitall()
+        # Wait for all uploads to finish
+        futures.wait(futures_list)
 
-        for part in plist:
-            pedict.update(part.etag)
-            total_size += part.size
+        # Final size validation after all parts uploaded
+        if image_size != 0 and total_bytes != image_size:
+            # Abort upload if mismatch (redundant check)
+            s3_client.abort_multipart_upload(
+                Bucket=bucket, Key=key, UploadId=upload_id)
+            raise glance_store.Invalid(
+                _("Size mismatch: expected %(expected)d "
+                  "bytes, got %(actual)d bytes") %
+                {'expected': image_size, 'actual': total_bytes})
 
-        success = True
-        for part in plist:
-            if not part.success:
-                success = False
+        # Check success status
+        success = all(p.success for p in plist)
+        total_size = sum(p.size for p in plist)
 
         if success:
             # Complete
+            pedict = {p.partnum: p.etag[p.partnum] for p in plist}
             mpu_list = self._get_mpu_list(pedict)
             s3_client.complete_multipart_upload(Bucket=bucket,
                                                 Key=key,
@@ -794,9 +888,8 @@ class Store(glance_store.driver.Store):
             raise exceptions.NotFound(image=key)
 
         LOG.debug("Deleting image object from S3 using s3_host=%(s3_host)s, "
-                  "accesskey=%(accesskey)s, bucket=%(bucket)s, key=%(key)s)",
-                  {'s3_host': loc.s3serviceurl, 'accesskey': loc.accesskey,
-                   'bucket': bucket, 'key': key})
+                  "bucket=%(bucket)s, key=%(key)s)",
+                  {'s3_host': loc.s3serviceurl, 'bucket': bucket, 'key': key})
 
         return s3_client.delete_object(Bucket=bucket, Key=key)
 
@@ -816,8 +909,7 @@ class Store(glance_store.driver.Store):
             error_code = e.response['Error']['Code']
             if error_code == '404':
                 return False
-            msg = ("Failed to get bucket info: %s" %
-                   encodeutils.exception_to_unicode(e))
+            msg = "Failed to get bucket info: %s" % e
             LOG.error(msg)
             raise glance_store.BadStoreConfiguration(store_name='s3',
                                                      reason=msg)
@@ -841,8 +933,7 @@ class Store(glance_store.driver.Store):
             error_code = e.response['Error']['Code']
             if error_code == '404':
                 return False
-            msg = ("Failed to get object info: %s" %
-                   encodeutils.exception_to_unicode(e))
+            msg = "Failed to get object info: %s" % e
             LOG.error(msg)
             raise glance_store.BadStoreConfiguration(store_name='s3',
                                                      reason=msg)
@@ -850,15 +941,20 @@ class Store(glance_store.driver.Store):
             return True
 
     @staticmethod
-    def _create_bucket(s3_client, s3_host, bucket):
+    def _create_bucket(s3_client, s3_host, bucket, region_name=None):
         """Create bucket into the S3.
 
         :param s3_client: An object with credentials to connect to S3
         :param s3_host: S3 endpoint url
         :param bucket: S3 bucket name
+        :param region_name: An optional region_name. If not provided, will try
+               to compute it from s3_host
         :raises: BadStoreConfiguration if cannot connect to S3 successfully
         """
-        region = get_s3_location(s3_host)
+        if region_name:
+            region = region_name
+        else:
+            region = get_s3_location(s3_host)
         try:
             s3_client.create_bucket(
                 Bucket=bucket,
@@ -869,8 +965,7 @@ class Store(glance_store.driver.Store):
                 }
             )
         except boto_exceptions.ClientError as e:
-            msg = ("Failed to add bucket to S3: %s" %
-                   encodeutils.exception_to_unicode(e))
+            msg = "Failed to add bucket to S3: %s" % e
             LOG.error(msg)
             raise glance_store.BadStoreConfiguration(store_name='s3',
                                                      reason=msg)
@@ -901,6 +996,7 @@ def get_s3_location(s3_host):
               Amazon S3, and if user wants to use S3 compatible storage,
               returns ''
     """
+    # NOTE(arnaud): maybe get rid of hardcoded amazon stuff here?
     locations = {
         's3.amazonaws.com': '',
         's3-us-east-1.amazonaws.com': 'us-east-1',
