@@ -12,11 +12,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import configparser
 import logging
-import sys
+import warnings
 
 from oslo_config import cfg
-from six.moves import configparser
 
 from glance_store import exceptions
 from glance_store.i18n import _, _LE
@@ -41,14 +41,14 @@ Related options:
     * None
 
 """),
-    cfg.StrOpt('swift_store_auth_version', default='2',
-               help='Version of the authentication service to use. '
-                    'Valid versions are 2 and 3 for keystone and 1 '
-                    '(deprecated) for swauth and rackspace.',
+    cfg.StrOpt('swift_store_auth_version', default='3',
+               choices=['3'],
+               help='The authentication version to be used. Currently '
+                    'The only valid version is 3.',
                deprecated_for_removal=True,
                deprecated_reason="""
-The option 'auth_version' in the Swift back-end configuration file is
-used instead.
+This option is kept for backword-compatibility reasons but is no longer
+required, because only the single version (3) is supported now.
 """),
     cfg.StrOpt('swift_store_auth_address',
                help='The address where the Swift authentication '
@@ -99,16 +99,11 @@ Related options:
 """),
 ]
 
-if sys.version_info >= (3, 2):
-    parser_class = configparser.ConfigParser
-else:
-    parser_class = configparser.SafeConfigParser
 
-
-class SwiftConfigParser(parser_class):
+class SwiftConfigParser(configparser.ConfigParser):
 
     def get(self, *args, **kwargs):
-        value = super(parser_class, self).get(*args, **kwargs)
+        value = super(configparser.ConfigParser, self).get(*args, **kwargs)
         return self._process_quotes(value)
 
     @staticmethod
@@ -122,10 +117,7 @@ class SwiftConfigParser(parser_class):
         return value
 
 
-if sys.version_info >= (3,):
-    CONFIG = SwiftConfigParser()
-else:
-    CONFIG = parser_class()
+CONFIG = SwiftConfigParser()
 
 LOG = logging.getLogger(__name__)
 
@@ -218,6 +210,11 @@ class SwiftParams(object):
 
                 try:
                     reference['auth_version'] = CONFIG.get(ref, 'auth_version')
+                    warnings.warn(
+                        'The auth_version option is deprecated. It is kept '
+                        'for backword-compatibility reasons but will be '
+                        'removed in a future release.',
+                        DeprecationWarning)
                 except configparser.NoOptionError:
                     if self.backend_group:
                         av = getattr(
@@ -227,6 +224,9 @@ class SwiftParams(object):
                         av = self.conf.glance_store.swift_store_auth_version
                     reference['auth_version'] = av
 
+                if reference['auth_version'] != '3':
+                    raise ValueError('Unsupported auth_version')
+
                 account_params[ref] = reference
             except (ValueError, SyntaxError, configparser.NoOptionError):
                 LOG.exception(_LE("Invalid format of swift store config cfg"))
