@@ -13,11 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import io
 from unittest import mock
 
 from oslo_config import cfg
 from oslo_utils import units
-import six
 
 import glance_store as store
 from glance_store._drivers import rbd as rbd_store
@@ -84,6 +84,9 @@ class MockRBD(object):
     class ImageHasSnapshots(Exception):
         pass
 
+    class IncompleteWriteError(Exception):
+        pass
+
     class ImageBusy(Exception):
         pass
 
@@ -113,6 +116,12 @@ class MockRBD(object):
         def remove_snap(self, *args, **kwargs):
             pass
 
+        def set_snap(self, *args, **kwargs):
+            pass
+
+        def list_children(self, *args, **kwargs):
+            pass
+
         def protect_snap(self, *args, **kwargs):
             pass
 
@@ -166,6 +175,9 @@ class MockRBD(object):
         def clone(self, *args, **kwargs):
             raise NotImplementedError()
 
+        def trash_move(self, *args, **kwargs):
+            pass
+
     RBD_FEATURE_LAYERING = 1
 
 
@@ -216,7 +228,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                                                 self.conf)
         # Provide enough data to get more than one chunk iteration.
         self.data_len = 3 * units.Ki
-        self.data_iter = six.BytesIO(b'*' * self.data_len)
+        self.data_iter = io.BytesIO(b'*' * self.data_len)
 
     def test_location_url_prefix_is_set(self):
         expected_url_prefix = "rbd://"
@@ -234,6 +246,47 @@ class TestMultiStore(base.MultiStoreBaseTest,
                 self.assertEqual(ret[1], self.data_len)
                 self.assertEqual("ceph1", ret[3]['store'])
 
+    @mock.patch.object(rbd_store.Store, '_delete_image')
+    def test_add_image_exceeding_max_size_raises_exception(self, mock_delete):
+        self.store.chunk_size = units.Ki
+        self.store.WRITE_CHUNKSIZE = 1024
+        total_bytes = units.Ki * 5
+        # actual data size
+        image_size = total_bytes
+        # Create data larger than image_size to simulate mismatch at end
+        data = b'a' * (total_bytes + 1024)
+        image_file = io.BytesIO(data)
+        with mock.patch.object(rbd_store.rbd.Image, 'write'):
+            self.assertRaisesRegex(
+                exceptions.Invalid, "Size exceeds: expected",
+                self.store.add, 'fake_image_id', image_file,
+                image_size)
+
+            # Confirm that image deletion was called due to size mismatch
+            mock_delete.assert_called()
+            # The position should be equal to total input size
+            self.assertEqual(image_file.tell(), len(data))
+
+    @mock.patch.object(rbd_store.Store, '_delete_image')
+    def test_write_less_than_declared_raises_exception(self, mock_delete):
+        self.store.chunk_size = units.Ki
+        total_bytes = units.Ki * 5
+        # actual data size
+        image_size = total_bytes
+        # Create data larger than image_size to simulate mismatch at end
+        data = b'a' * (total_bytes - 100)
+        image_file = io.BytesIO(data)
+        with mock.patch.object(rbd_store.rbd.Image, 'write'):
+            self.assertRaisesRegex(
+                exceptions.Invalid, "Size mismatch: expected",
+                self.store.add, 'fake_image_id', image_file,
+                image_size)
+
+            # Confirm that image deletion was called due to size mismatch
+            mock_delete.assert_called()
+            # The position should be less than total input size
+            self.assertLessEqual(image_file.tell(), len(data))
+
     def test_add_w_image_size_zero_to_different_backend(self):
         """Assert that correct size is returned even though 0 was provided."""
         self.store = rbd_store.Store(self.conf, backend="ceph2")
@@ -247,7 +300,7 @@ class TestMultiStore(base.MultiStoreBaseTest,
                                                 self.conf)
         # Provide enough data to get more than one chunk iteration.
         self.data_len = 3 * units.Ki
-        self.data_iter = six.BytesIO(b'*' * self.data_len)
+        self.data_iter = io.BytesIO(b'*' * self.data_len)
         self.store.chunk_size = units.Ki
         with mock.patch.object(rbd_store.rbd.Image, 'resize') as resize:
             with mock.patch.object(rbd_store.rbd.Image, 'write') as write:
@@ -417,6 +470,14 @@ class TestMultiStore(base.MultiStoreBaseTest,
 
             self.called_commands_expected = ['unprotect_snap']
 
+    def test_delete_image_snap_has_external_references(self):
+        with mock.patch.object(MockRBD.Image, 'list_children') as mocked:
+            mocked.return_value = True
+
+            self.store._delete_image('fake_pool',
+                                     self.location.image,
+                                     snapshot_name='snap')
+
     def test_delete_image_w_snap_exc_image_has_snap(self):
         def _fake_remove(*args, **kwargs):
             self.called_commands_actual.append('remove')
@@ -424,8 +485,8 @@ class TestMultiStore(base.MultiStoreBaseTest,
 
         with mock.patch.object(MockRBD.RBD, 'remove') as remove:
             remove.side_effect = _fake_remove
-            self.assertRaises(exceptions.HasSnapshot, self.store._delete_image,
-                              'fake_pool', self.location.image)
+            self.store._delete_image('fake_pool',
+                                     self.location.image)
 
             self.called_commands_expected = ['remove']
 
@@ -435,14 +496,6 @@ class TestMultiStore(base.MultiStoreBaseTest,
         self.assertRaises(exceptions.StoreRandomGetNotSupported,
                           self.store.get, loc, chunk_size=1)
 
-    @mock.patch.object(MockRados.Rados, 'connect')
-    def test_rados_connect_timeout(self, mock_rados_connect):
-        socket_timeout = 1
-        self.config(rados_connect_timeout=socket_timeout, group="ceph1")
-        self.store.configure()
-        with self.store.get_connection('conffile', 'rados_id'):
-            mock_rados_connect.assert_called_with(timeout=socket_timeout)
-
     @mock.patch.object(MockRados.Rados, 'connect', side_effect=MockRados.Error)
     def test_rados_connect_error(self, _):
         rbd_store.rados.Error = MockRados.Error
