@@ -77,11 +77,14 @@ class OptsTestCase(base.StoreBaseTest):
             'cinder_state_transition_timeout',
             'cinder_store_auth_address',
             'cinder_store_user_name',
+            'cinder_store_user_domain_name',
             'cinder_store_password',
             'cinder_store_project_name',
+            'cinder_store_project_domain_name',
             'cinder_volume_type',
             'cinder_use_multipath',
             'cinder_enforce_multipath',
+            'cinder_do_extend_attached',
             'default_swift_reference',
             'https_insecure',
             'filesystem_store_chunk_size',
@@ -104,10 +107,12 @@ class OptsTestCase(base.StoreBaseTest):
             's3_store_bucket_url_format',
             's3_store_create_bucket_on_put',
             's3_store_host',
+            's3_store_region_name',
             's3_store_secret_key',
             's3_store_large_object_size',
             's3_store_large_object_chunk_size',
             's3_store_thread_pools',
+            's3_store_cacert',
             'swift_store_expire_soon_interval',
             'swift_store_admin_tenants',
             'swift_store_auth_address',
